# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: econ_gcmessages.proto
# plugin: python-betterproto

from dataclasses import dataclass

import betterproto

from ....protobufs.msg import GCProtobufMessage
from ..enums import EMsg


@dataclass(eq=False, repr=False)
class GiftedItems(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    giftdefindex: int = betterproto.uint32_field(2)
    max_gifts_possible: int = betterproto.uint32_field(3)
    num_eligible_recipients: int = betterproto.uint32_field(4)
    recipients_accountids: list[int] = betterproto.uint32_field(5)


@dataclass(eq=False, repr=False)
class ApplyAutograph(betterproto.Message):
    autograph_item_id: int = betterproto.uint64_field(1)
    item_item_id: int = betterproto.uint64_field(2)


class CasketItemAdd(GCProtobufMessage, msg=EMsg.CasketItemAdd):
    casket_item_id: int = betterproto.uint64_field(1)
    item_item_id: int = betterproto.uint64_field(2)


class CasketItemExtract(GCProtobufMessage, msg=EMsg.CasketItemExtract):
    casket_item_id: int = betterproto.uint64_field(1)
    item_item_id: int = betterproto.uint64_field(2)


class CasketItemLoadContents(GCProtobufMessage, msg=EMsg.CasketItemLoadContents):
    casket_item_id: int = betterproto.uint64_field(1)
    item_item_id: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class UserTrackTimePlayedConsecutively(betterproto.Message):
    state: int = betterproto.uint32_field(1)


class ItemCustomizationNotification(GCProtobufMessage, msg=EMsg.ItemCustomizationNotification):
    item_id: list[int] = betterproto.uint64_field(1)
    request: int = betterproto.uint32_field(2)
