# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: gcsystemmsgs.proto
# plugin: python-betterproto

from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class GamePersonalDataCategoryInfo(betterproto.Message):
    type: str = betterproto.string_field(1)
    localization_token: str = betterproto.string_field(2)
    template_file: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class GetGamePersonalDataCategoriesRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class GetGamePersonalDataCategoriesResponse(betterproto.Message):
    categories: "list[GamePersonalDataCategoryInfo]" = betterproto.message_field(1)
    app_assets_basename: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class GetGamePersonalDataEntriesRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    steamid: int = betterproto.uint64_field(2)
    type: str = betterproto.string_field(3)
    continue_token: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class GetGamePersonalDataEntriesResponse(betterproto.Message):
    gceresult: int = betterproto.uint32_field(1)
    entries: list[str] = betterproto.string_field(2)
    continue_token: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class TerminateGamePersonalDataEntriesRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    steamid: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class TerminateGamePersonalDataEntriesResponse(betterproto.Message):
    gceresult: int = betterproto.uint32_field(1)
