# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: gcsystemmsgs.proto
# plugin: python-betterproto

from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class CMsgGchVacVerificationChange(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)
    appid: int = betterproto.uint32_field(2)
    is_verified: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CMsgGchAccountPhoneNumberChange(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)
    appid: int = betterproto.uint32_field(2)
    phone_id: int = betterproto.uint64_field(3)
    is_verified: bool = betterproto.bool_field(4)
    is_identifying: bool = betterproto.bool_field(5)


@dataclass(eq=False, repr=False)
class CMsgGchInviteUserToLobby(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)
    appid: int = betterproto.uint32_field(2)
    steamid_invited: int = betterproto.fixed64_field(3)
    steamid_lobby: int = betterproto.fixed64_field(4)


@dataclass(eq=False, repr=False)
class CQuestPublisherAddCommunityItemsToPlayerRequest(betterproto.Message):
    steamid: int = betterproto.uint64_field(1)
    appid: int = betterproto.uint32_field(2)
    match_item_type: int = betterproto.uint32_field(3)
    match_item_class: int = betterproto.uint32_field(4)
    prefix_item_name: str = betterproto.string_field(5)
    attributes: "list[CQuestPublisherAddCommunityItemsToPlayerRequestAttribute]" = betterproto.message_field(6)
    note: str = betterproto.string_field(7)


@dataclass(eq=False, repr=False)
class CQuestPublisherAddCommunityItemsToPlayerRequestAttribute(betterproto.Message):
    attribute: int = betterproto.uint32_field(1)
    value: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CQuestPublisherAddCommunityItemsToPlayerResponse(betterproto.Message):
    items_matched: int = betterproto.uint32_field(1)
    items_granted: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CCommunityGamePersonalDataCategoryInfo(betterproto.Message):
    type: str = betterproto.string_field(1)
    localization_token: str = betterproto.string_field(2)
    template_file: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CCommunityGetGamePersonalDataCategoriesRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class CCommunityGetGamePersonalDataCategoriesResponse(betterproto.Message):
    categories: "list[CCommunityGamePersonalDataCategoryInfo]" = betterproto.message_field(1)
    app_assets_basename: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CCommunityGetGamePersonalDataEntriesRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    steamid: int = betterproto.uint64_field(2)
    type: str = betterproto.string_field(3)
    continue_token: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class CCommunityGetGamePersonalDataEntriesResponse(betterproto.Message):
    gceresult: int = betterproto.uint32_field(1)
    entries: list[str] = betterproto.string_field(2)
    continue_token: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CCommunityTerminateGamePersonalDataEntriesRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    steamid: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CCommunityTerminateGamePersonalDataEntriesResponse(betterproto.Message):
    gceresult: int = betterproto.uint32_field(1)
